/*
 * Single linked list.
 *
 * This software is distributed under the terms of GNU
 * Public License version 2 or later versions.
 *
 */

#ifndef _SLIST_H
#define _SLIST_H

struct slist {
	struct slist	*next;
	void		*data;
};

typedef int (*slist_compare_t)(const void *a, const void *b);

#define slist_foreach(list, iter) \
	for ((iter) = (list); (iter) != NULL; (iter) = (iter)->next)

#define slist_first(list)	((list)->next)
#define slist_is_empty(list)	((list) == NULL)

/* Free all elements of the list. */
void slist_free(struct slist **list);

/* Add element to the list. */
struct slist *slist_add(struct slist *list, struct slist *e);

/* Add element to the tail of the list. */
struct slist *slist_add_tail(struct slist *list, struct slist *e);

/* Add data to to the list. */
struct slist *slist_add_data(struct slist *list, void *data);

/* Delete data from the list. */
struct slist *slist_del_data(struct slist *list, void *data);

/* Reverse order of elements in the list. */
struct slist *slist_reverse(struct slist *list);

/* Unlink element from the list. */
struct slist *slist_unlink(struct slist *list, struct slist *e);

/* Pull out the first element from the list head. */
struct slist *slist_extract(struct slist **list);

/* Get pointer to n-th element of the list. */
struct slist *slist_nth(struct slist *list, unsigned int nth);

/* Extract n-th element from the list. */
struct slist *slist_extract_nth(struct slist **list, unsigned int nth);

/* Sort list using compare function. */
struct slist *slist_sort(struct slist *list, slist_compare_t compare);

/* Returns number of elements in the list. */
size_t slist_count(struct slist *list);

#endif
